@@ -1502,13 +1502,14 @@ int rdp_recv_callback(rdpTransport* transport, wStream* s, void* extra)
 	 * enters the active state, an auto-detect PDU can be received
 	 * on the MCS message channel.
 	 */
-	if ((rdp->state > CONNECTION_STATE_MCS_CHANNEL_JOIN) && (rdp->state < CONNECTION_STATE_ACTIVE))
+	if ((rdp_get_state(rdp) > CONNECTION_STATE_MCS_CHANNEL_JOIN) &&
+	    (rdp_get_state(rdp) < CONNECTION_STATE_ACTIVE))
 	{
 		if (rdp_client_connect_auto_detect(rdp, s))
 			return 0;
 	}
 
-	switch (rdp->state)
+	switch (rdp_get_state(rdp))
 	{
 		case CONNECTION_STATE_NLA:
 			if (nla_get_state(rdp->nla) < NLA_STATE_AUTH_INFO)
@@ -1516,7 +1517,7 @@ int rdp_recv_callback(rdpTransport* transport, wStream* s, void* extra)
 				if (nla_recv_pdu(rdp->nla, s) < 1)
 				{
 					WLog_ERR(TAG, "%s: %s - nla_recv_pdu() fail", __FUNCTION__,
-					         rdp_server_connection_state_string(rdp->state));
+					         rdp_state_string(rdp_get_state(rdp)));
 					return -1;
 				}
 			}
@@ -1527,7 +1528,7 @@ int rdp_recv_callback(rdpTransport* transport, wStream* s, void* extra)
 				if (nego_get_state(rdp->nego) != NEGO_STATE_FINAL)
 				{
 					WLog_ERR(TAG, "%s: %s - nego_recv() fail", __FUNCTION__,
-					         rdp_server_connection_state_string(rdp->state));
+					         rdp_state_string(rdp_get_state(rdp)));
 					return -1;
 				}
 
@@ -1567,7 +1568,7 @@ int rdp_recv_callback(rdpTransport* transport, wStream* s, void* extra)
 				if (!mcs_client_begin(rdp->mcs))
 				{
 					WLog_ERR(TAG, "%s: %s - mcs_client_begin() fail", __FUNCTION__,
-					         rdp_server_connection_state_string(rdp->state));
+					         rdp_state_string(rdp_get_state(rdp)));
 					return -1;
 				}
 			}
@@ -1618,7 +1619,7 @@ int rdp_recv_callback(rdpTransport* transport, wStream* s, void* extra)
 				WLog_ERR(TAG,
 				         "%s: %s - "
 				         "rdp_client_connect_mcs_channel_join_confirm() fail",
-				         __FUNCTION__, rdp_server_connection_state_string(rdp->state));
+				         __FUNCTION__, rdp_state_string(rdp_get_state(rdp)));
 				status = -1;
 			}
 
@@ -1629,7 +1630,7 @@ int rdp_recv_callback(rdpTransport* transport, wStream* s, void* extra)
 
 			if (status < 0)
 				WLog_DBG(TAG, "%s: %s - rdp_client_connect_license() - %i", __FUNCTION__,
-				         rdp_server_connection_state_string(rdp->state), status);
+				         rdp_state_string(rdp_get_state(rdp)), status);
 
 			break;
 
@@ -1640,7 +1641,7 @@ int rdp_recv_callback(rdpTransport* transport, wStream* s, void* extra)
 				WLog_DBG(TAG,
 				         "%s: %s - "
 				         "rdp_client_connect_demand_active() - %i",
-				         __FUNCTION__, rdp_server_connection_state_string(rdp->state), status);
+				         __FUNCTION__, rdp_state_string(rdp_get_state(rdp)), status);
 
 			break;
 
@@ -1655,7 +1656,7 @@ int rdp_recv_callback(rdpTransport* transport, wStream* s, void* extra)
 
 			if (status < 0)
 				WLog_DBG(TAG, "%s: %s - rdp_recv_pdu() - %i", __FUNCTION__,
-				         rdp_server_connection_state_string(rdp->state), status);
+				         rdp_state_string(rdp_get_state(rdp)), status);
 
 			break;
 
@@ -1664,13 +1665,13 @@ int rdp_recv_callback(rdpTransport* transport, wStream* s, void* extra)
 
 			if (status < 0)
 				WLog_DBG(TAG, "%s: %s - rdp_recv_pdu() - %i", __FUNCTION__,
-				         rdp_server_connection_state_string(rdp->state), status);
+				         rdp_state_string(rdp_get_state(rdp)), status);
 
 			break;
 
 		default:
-			WLog_ERR(TAG, "%s: %s state %d", __FUNCTION__,
-			         rdp_server_connection_state_string(rdp->state), rdp->state);
+			WLog_ERR(TAG, "%s: %s state %d", __FUNCTION__, rdp_state_string(rdp_get_state(rdp)),
+			         rdp_get_state(rdp));
 			status = -1;
 			break;
 	}
